@@ -26,14 +26,15 @@
 #include "mstyle.h"
 #include "gnm-style-impl.h"
 #include "expr.h"
+#include "expr-impl.h"
 #include "cell.h"
 #include "value.h"
 #include "sheet.h"
 #include <parse-util.h>
 #include <gsf/gsf-impl-utils.h>
 #include <string.h>
-#include <parse-util.h>
-#include "gutils.h"
+#include <func.h>
+#include <gutils.h>
 
 #define BLANKS_STRING_FOR_MATCHING " \t\n\r"
 
@@ -241,6 +242,394 @@ gnm_style_cond_set_overlay (GnmStyleCond *cond, GnmStyle *overlay)
 	cond->overlay = overlay;
 }
 
+static GnmExpr const *
+generate_end_match (const char *endfunc, gboolean force,
+		    GnmExprTop const *sexpr, GnmCellRef *cr)
+{
+	GnmValue const *v = gnm_expr_get_constant (sexpr->expr);
+	GnmExpr const *len_expr;
+
+	if (v && VALUE_IS_STRING (v)) {
+		int len = g_utf8_strlen (value_peek_string (v), -1);
+		len_expr = gnm_expr_new_constant (value_new_int (len));
+	} else if (force) {
+		/*
+		 * This is imperfect because the expression gets
+		 * evaluated twice.
+		 */
+		len_expr = gnm_expr_new_funcall1
+			(gnm_func_lookup_or_add_placeholder ("LEN"),
+			 gnm_expr_copy (sexpr->expr));
+	} else
+		return NULL;
+
+	return gnm_expr_new_funcall2
+		(gnm_func_lookup_or_add_placeholder ("EXACT"),
+		 gnm_expr_new_funcall2
+		 (gnm_func_lookup_or_add_placeholder (endfunc),
+		  gnm_expr_new_cellref (cr),
+		  len_expr),
+		 gnm_expr_copy (sexpr->expr));
+}
+
+
+/**
+ * gnm_style_cond_get_alternate_expr:
+ * @cond: condition
+ *
+ * Returns: (transfer full) (allow-none): An custom expression that can be
+ * used in place of @cond.
+ **/
+GnmExprTop const *
+gnm_style_cond_get_alternate_expr (GnmStyleCond const *cond)
+{
+	GnmCellRef self;
+	GnmExpr const *expr;
+	gboolean negate = FALSE;
+	GnmExprTop const *sexpr = NULL;
+
+	g_return_val_if_fail (cond != NULL, NULL);
+
+	gnm_cellref_init (&self, NULL, 0, 0, TRUE);
+
+	if (gnm_style_cond_op_operands (cond->op) > 0) {
+		sexpr = gnm_style_cond_get_expr (cond, 0);
+		if (!sexpr)
+			return NULL;
+	}
+
+	switch (cond->op) {
+	case GNM_STYLE_COND_NOT_CONTAINS_ERR:
+		negate = TRUE; /* ...and fall through */
+	case GNM_STYLE_COND_CONTAINS_ERR:
+		expr = gnm_expr_new_funcall1
+			(gnm_func_lookup_or_add_placeholder ("ISERROR"),
+			 gnm_expr_new_cellref (&self));
+		break;
+
+	case GNM_STYLE_COND_CONTAINS_STR:
+		negate = TRUE; /* ...and fall through */
+	case GNM_STYLE_COND_NOT_CONTAINS_STR:
+		expr = gnm_expr_new_funcall1
+			(gnm_func_lookup_or_add_placeholder ("ISERROR"),
+			 gnm_expr_new_funcall2
+			 (gnm_func_lookup_or_add_placeholder ("FIND"),
+			  gnm_expr_copy (sexpr->expr),
+			  gnm_expr_new_cellref (&self)));
+		break;
+
+	case GNM_STYLE_COND_CONTAINS_BLANKS:
+		negate = TRUE; /* ...and fall through */
+	case GNM_STYLE_COND_NOT_CONTAINS_BLANKS:
+		/* This is imperfect.  Here we only look for space.  */
+		expr = gnm_expr_new_funcall1
+			(gnm_func_lookup_or_add_placeholder ("ISERROR"),
+			 gnm_expr_new_funcall2
+			 (gnm_func_lookup_or_add_placeholder ("FIND"),
+			  gnm_expr_new_constant (value_new_string (" ")),
+			  gnm_expr_new_cellref (&self)));
+		break;
+
+	case GNM_STYLE_COND_NOT_BEGINS_WITH_STR:
+		negate = TRUE; /* ...and fall through */
+	case GNM_STYLE_COND_BEGINS_WITH_STR:
+		expr = generate_end_match ("LEFT", FALSE, sexpr, &self);
+		if (expr)
+			break;
+
+		/*
+		 * We get here if sexpr is not a constant.  We are
+		 * constrained by using only Excel functions and not
+		 * evaluating the needle more than once.  The
+		 * expression used here works, but FIND is more
+		 * expensive that it ought to be.
+		 */
+		expr = gnm_expr_new_binary
+			(gnm_expr_new_funcall2
+			 (gnm_func_lookup_or_add_placeholder ("IFERROR"),
+			  gnm_expr_new_funcall2
+			  (gnm_func_lookup_or_add_placeholder ("FIND"),
+			   gnm_expr_copy (sexpr->expr),
+			   gnm_expr_new_cellref (&self)),
+			  gnm_expr_new_constant (value_new_int (2))),
+			 GNM_EXPR_OP_EQUAL,
+			 gnm_expr_new_constant (value_new_int (1)));
+		break;
+
+	case GNM_STYLE_COND_NOT_ENDS_WITH_STR:
+		negate = TRUE; /* ...and fall through */
+	case GNM_STYLE_COND_ENDS_WITH_STR:
+		/*
+		 * We are constrained by using only Excel functions and not
+		 * evaluating the needle more than once.  We cannot fulfill
+		 * that and end up computing the needle twice.
+		 */
+		expr = generate_end_match ("RIGHT", TRUE, sexpr, &self);
+		break;
+
+	default:
+		return NULL;
+	}
+
+	if (negate)
+		expr = gnm_expr_new_funcall1
+			(gnm_func_lookup_or_add_placeholder ("NOT"), expr);
+
+	return gnm_expr_top_new (expr);
+}
+
+static gboolean
+isself (GnmExpr const *expr)
+{
+	GnmCellRef const *cr;
+
+	if (GNM_EXPR_GET_OPER (expr) != GNM_EXPR_OP_CELLREF)
+		return FALSE;
+	cr = &expr->cellref.ref;
+
+	return (cr->sheet == NULL &&
+		cr->col == 0 && cr->row == 0 &&
+		cr->col_relative && cr->row_relative);
+}
+
+static GnmExprTop const *
+decode_end_match (const char *endfunc, GnmExpr const *expr)
+{
+	GnmExpr const *needle;
+	GnmExpr const *expr2;
+
+	if (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_FUNCALL &&
+	    expr->func.argc == 2 &&
+	    expr->func.func == gnm_func_lookup_or_add_placeholder ("EXACT") &&
+	    (needle = expr->func.argv[1]) &&
+	    (expr2 = expr->func.argv[0]) &&
+	    GNM_EXPR_GET_OPER (expr2) == GNM_EXPR_OP_FUNCALL &&
+	    expr2->func.argc == 2 &&
+	    expr2->func.func == gnm_func_lookup_or_add_placeholder (endfunc) &&
+	    isself (expr2->func.argv[0])) {
+		GnmExpr const *len_expr = expr2->func.argv[1];
+		GnmValue const *v, *vl;
+
+		if (GNM_EXPR_GET_OPER (len_expr) == GNM_EXPR_OP_FUNCALL &&
+		    len_expr->func.argc == 1 &&
+		    len_expr->func.func == gnm_func_lookup_or_add_placeholder ("LEN") &&
+		    gnm_expr_equal (len_expr->func.argv[0], needle))
+			return gnm_expr_top_new (gnm_expr_copy (needle));
+
+		if ((v = gnm_expr_get_constant (needle)) &&
+		    VALUE_IS_STRING (v) &&
+		    (vl = gnm_expr_get_constant (len_expr)) &&
+		    VALUE_IS_NUMBER (vl) &&
+		    value_get_as_float (vl) == g_utf8_strlen (value_peek_string (v), -1))
+			return gnm_expr_top_new (gnm_expr_copy (needle));
+	}
+
+	return NULL;
+}
+
+/**
+ * gnm_style_cond_canonicalize:
+ * @cond: condition
+ *
+ * Turns a custom condition into a more specific one, i.e., reverses the
+ * effect of using gnm_style_cond_get_alternate_expr.  Leaves the condition
+ * alone if it is not recognized.
+ **/
+void
+gnm_style_cond_canonicalize (GnmStyleCond *cond)
+{
+	GnmExpr const *expr, *expr2;
+	GnmExprTop const *texpr;
+	GnmValue const *v;
+	gboolean negate = FALSE;
+	GnmFunc const *iserror;
+	GnmFunc const *iferror;
+	GnmFunc const *find;
+	GnmStyleCondOp newop = GNM_STYLE_COND_CUSTOM;
+
+	g_return_if_fail (cond != NULL);
+
+	if (cond->op != GNM_STYLE_COND_CUSTOM)
+		return;
+
+	texpr = gnm_style_cond_get_expr (cond, 0);
+	if (!texpr)
+		return;
+	expr = texpr->expr;
+	texpr = NULL;
+
+	if (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_FUNCALL &&
+	    expr->func.argc == 1 &&
+	    expr->func.func == gnm_func_lookup_or_add_placeholder ("NOT")) {
+		negate = TRUE;
+		expr = expr->func.argv[0];
+	}
+
+	iserror = gnm_func_lookup_or_add_placeholder ("ISERROR");
+	iferror = gnm_func_lookup_or_add_placeholder ("IFERROR");
+	find = gnm_func_lookup_or_add_placeholder ("FIND");
+
+	if (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_FUNCALL &&
+	    expr->func.argc == 1 && expr->func.func == iserror &&
+	    isself (expr->func.argv[0])) {
+		newop = negate
+			? GNM_STYLE_COND_NOT_CONTAINS_ERR
+			: GNM_STYLE_COND_CONTAINS_ERR;
+	} else if (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_FUNCALL &&
+		   expr->func.argc == 1 && expr->func.func == iserror &&
+		   (expr2 = expr->func.argv[0]) &&
+		   GNM_EXPR_GET_OPER (expr2) == GNM_EXPR_OP_FUNCALL &&
+		   expr2->func.argc == 2 && expr2->func.func == find &&
+		   isself (expr2->func.argv[1])) {
+		GnmValue const *v =
+			gnm_expr_get_constant (expr2->func.argv[0]);
+
+		if (v && VALUE_IS_STRING (v) &&
+		    strcmp (value_peek_string (v), " ") == 0)
+			newop = negate
+				? GNM_STYLE_COND_CONTAINS_BLANKS
+				: GNM_STYLE_COND_NOT_CONTAINS_BLANKS;
+		else {
+			texpr = gnm_expr_top_new (gnm_expr_copy (expr2->func.argv[0]));
+			newop = negate
+				? GNM_STYLE_COND_CONTAINS_STR
+				: GNM_STYLE_COND_NOT_CONTAINS_STR;
+		}
+	} else if (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_EQUAL &&
+		   (v = gnm_expr_get_constant (expr->binary.value_b)) &&
+		   VALUE_IS_FLOAT (v) && value_get_as_float (v) == 1 &&
+		   (expr2 = expr->binary.value_a) &&
+		   GNM_EXPR_GET_OPER (expr2) == GNM_EXPR_OP_FUNCALL &&
+		   expr2->func.argc == 2 && expr2->func.func == iferror &&
+		   (v = gnm_expr_get_constant (expr2->func.argv[1])) &&
+		   VALUE_IS_FLOAT (v) && value_get_as_float (v) != 1 &&
+		   (expr2 = expr2->func.argv[0]) &&
+		   GNM_EXPR_GET_OPER (expr2) == GNM_EXPR_OP_FUNCALL &&
+		   expr2->func.argc == 2 && expr2->func.func == find &&
+		   isself (expr2->func.argv[1])) {
+		texpr = gnm_expr_top_new (gnm_expr_copy (expr2->func.argv[0]));
+		newop = negate
+			? GNM_STYLE_COND_NOT_BEGINS_WITH_STR
+			: GNM_STYLE_COND_BEGINS_WITH_STR;
+	} else if ((texpr = decode_end_match ("LEFT", expr))) {
+		newop = negate
+			? GNM_STYLE_COND_NOT_BEGINS_WITH_STR
+			: GNM_STYLE_COND_BEGINS_WITH_STR;
+	} else if ((texpr = decode_end_match ("RIGHT", expr))) {
+		newop = negate
+			? GNM_STYLE_COND_NOT_ENDS_WITH_STR
+			: GNM_STYLE_COND_ENDS_WITH_STR;
+	}
+
+	if (newop != GNM_STYLE_COND_CUSTOM) {
+		gnm_style_cond_set_expr (cond, texpr, 0);
+		if (texpr)
+			gnm_expr_top_unref (texpr);
+		cond->op = newop;
+	}
+}
+
+
+static gboolean
+gnm_style_cond_eval (GnmStyleCond const *cond, GnmValue const *cv,
+		     GnmEvalPos const *ep)
+{
+	gboolean negate = FALSE;
+	gboolean res;
+	GnmValue *val0 = NULL;
+	GnmValue *val1 = NULL;
+
+	switch (gnm_style_cond_op_operands (cond->op)) {
+	case 2:
+		val1 = gnm_expr_top_eval (cond->deps[1].texpr, ep,
+					  GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
+		/* Fall through */
+	case 1:
+		val0 = gnm_expr_top_eval (cond->deps[0].texpr, ep,
+					  GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
+		/* Fall through */
+	case 0:
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+
+	switch (cond->op) {
+	case GNM_STYLE_COND_NOT_EQUAL:
+		negate = TRUE;  /* ...and fall through */
+	case GNM_STYLE_COND_EQUAL:
+		res = value_compare (cv, val0, FALSE) == IS_EQUAL;
+		break;
+
+	case GNM_STYLE_COND_LTE:
+		negate = TRUE;  /* ...and fall through */
+	case GNM_STYLE_COND_GT:
+		res = value_compare (cv, val0, FALSE) == IS_GREATER;
+		break;
+		
+	case GNM_STYLE_COND_GTE:
+		negate = TRUE;  /* ...and fall through */
+	case GNM_STYLE_COND_LT:
+		res = value_compare (cv, val0, FALSE) == IS_LESS;
+		break;
+
+	case GNM_STYLE_COND_NOT_BETWEEN:
+		negate = TRUE;  /* ...and fall through */
+	case GNM_STYLE_COND_BETWEEN:
+		res = !(value_compare (cv, val0, FALSE) == IS_LESS ||
+			value_compare (cv, val1, FALSE) == IS_GREATER);
+		break;
+
+	case GNM_STYLE_COND_NOT_CONTAINS_ERR:
+		negate = TRUE;  /* ...and fall through */
+	case GNM_STYLE_COND_CONTAINS_ERR:
+		res = cv && VALUE_IS_ERROR (cv);
+		break;
+
+	case GNM_STYLE_COND_NOT_CONTAINS_BLANKS:
+		negate = TRUE;  /* ...and fall through */
+	case GNM_STYLE_COND_CONTAINS_BLANKS:
+		res = (cv &&
+		       strpbrk (value_peek_string (cv), BLANKS_STRING_FOR_MATCHING) != NULL);
+		break;
+
+	case GNM_STYLE_COND_NOT_CONTAINS_STR:
+		negate = TRUE;  /* ...and fall through */
+	case GNM_STYLE_COND_CONTAINS_STR:
+		res = (cv &&
+		       strstr (value_peek_string (cv),
+			       value_peek_string (val0)) != NULL);
+		break;
+
+	case GNM_STYLE_COND_NOT_BEGINS_WITH_STR:
+		negate = TRUE;  /* ...and fall through */
+	case GNM_STYLE_COND_BEGINS_WITH_STR:
+		res = (cv &&
+		       g_str_has_prefix (value_peek_string (cv),
+					 value_peek_string (val0)));
+		break;
+
+	case GNM_STYLE_COND_NOT_ENDS_WITH_STR:
+		negate = TRUE;  /* ...and fall through */
+	case GNM_STYLE_COND_ENDS_WITH_STR:
+		res = (cv &&
+		       g_str_has_suffix (value_peek_string (cv),
+					 value_peek_string (val0)));
+		break;
+
+	case GNM_STYLE_COND_CUSTOM:
+		res = value_get_as_bool (val0, NULL);
+		break;
+
+	default:
+		g_assert_not_reached ();
+	}
+
+	value_release (val0);
+	value_release (val1);
+
+	return negate ? !res : res;
+}
 
 static void
 gnm_style_conditions_finalize (GObject *obj)
@@ -493,10 +882,7 @@ int
 gnm_style_conditions_eval (GnmStyleConditions const *sc, GnmEvalPos const *ep)
 {
 	unsigned i;
-	gboolean use_this = FALSE;
-	GnmValue *val = NULL;
 	GPtrArray const *conds;
-	GnmParsePos pp;
 	GnmCell const *cell = sheet_cell_get (ep->sheet, ep->eval.col, ep->eval.row);
 	GnmValue const *cv = cell ? cell->value : NULL;
 
@@ -504,9 +890,11 @@ gnm_style_conditions_eval (GnmStyleConditions const *sc, GnmEvalPos const *ep)
 	g_return_val_if_fail (sc->conditions != NULL, -1);
 
 	conds = sc->conditions;
-	parse_pos_init_evalpos (&pp, ep);
 
 	if (debug_style_conds ()) {
+		GnmParsePos pp;
+		parse_pos_init_evalpos (&pp, ep);
+
 		g_printerr ("Evaluating conditions %p at %s with %d clauses\n",
 			    sc,
 			    parsepos_as_string (&pp),
@@ -515,94 +903,7 @@ gnm_style_conditions_eval (GnmStyleConditions const *sc, GnmEvalPos const *ep)
 
 	for (i = 0 ; i < conds->len ; i++) {
 		GnmStyleCond const *cond = g_ptr_array_index (conds, i);
-
-		if (cond->op == GNM_STYLE_COND_CONTAINS_ERR)
-			use_this = (cv != NULL) && VALUE_IS_ERROR (cv);
-		else if (cond->op == GNM_STYLE_COND_NOT_CONTAINS_ERR)
-			use_this = (cv == NULL) || !VALUE_IS_ERROR (cv);
-		else if (cond->op == GNM_STYLE_COND_CONTAINS_BLANKS ||
-			 cond->op == GNM_STYLE_COND_NOT_CONTAINS_BLANKS) {
-			if (cv && VALUE_IS_STRING (cv)) {
-				char const *cvstring = value_peek_string (cv);
-				switch (cond->op) {
-				case GNM_STYLE_COND_CONTAINS_BLANKS :
-					use_this = NULL != strpbrk (cvstring, BLANKS_STRING_FOR_MATCHING);
-					break;
-				case GNM_STYLE_COND_NOT_CONTAINS_BLANKS :
-					use_this = NULL == strpbrk (cvstring, BLANKS_STRING_FOR_MATCHING);
-					break;
-				default:
-					break;
-				}
-			}
-		} else {
-			val = gnm_expr_top_eval (cond->deps[0].texpr, ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
-			if (cond->op == GNM_STYLE_COND_CUSTOM) {
-				use_this = value_get_as_bool (val, NULL);
-#if 0
-				char *str = gnm_expr_as_string (cond->expr[0],
-								&pp, NULL);
-				g_print ("'%s' = %s\n", str, use_this ? "true" : "false");
-				g_free (str);
-#endif
-			} else if (cond->op < GNM_STYLE_COND_CONTAINS_STR) {
-				GnmValDiff diff = value_compare (cv, val, TRUE);
-
-				switch (cond->op) {
-				default:
-				case GNM_STYLE_COND_EQUAL:	use_this = (diff == IS_EQUAL); break;
-				case GNM_STYLE_COND_NOT_EQUAL:	use_this = (diff != IS_EQUAL); break;
-				case GNM_STYLE_COND_NOT_BETWEEN:
-					if (diff == IS_LESS) {
-						use_this = TRUE;
-						break;
-					}
-					value_release (val);
-					val = gnm_expr_top_eval (cond->deps[1].texpr, ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
-					diff = value_compare (cv, val, TRUE);
-					/* fall through */
-
-				case GNM_STYLE_COND_GT:		use_this = (diff == IS_GREATER); break;
-				case GNM_STYLE_COND_LT:		use_this = (diff == IS_LESS); break;
-				case GNM_STYLE_COND_GTE:	use_this = (diff == IS_GREATER || diff == IS_EQUAL); break;
-				case GNM_STYLE_COND_BETWEEN:
-					if (diff == IS_LESS)
-						break;
-					value_release (val);
-					val = gnm_expr_top_eval (cond->deps[1].texpr, ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
-					diff = value_compare (cv, val, TRUE);
-					/* fall through */
-				case GNM_STYLE_COND_LTE:	use_this = (diff == IS_LESS || diff == IS_EQUAL); break;
-				}
-			} else if (cv && VALUE_IS_STRING (cv)) {
-				char const *valstring = value_peek_string (val);
-				char const *cvstring = value_peek_string (cv);
-
-				switch (cond->op) {
-				default : g_warning ("Unknown condition operator %d", cond->op);
-					break;
-				case GNM_STYLE_COND_CONTAINS_STR :
-					use_this = (NULL != strstr (cvstring, valstring));
-					break;
-				case GNM_STYLE_COND_NOT_CONTAINS_STR :
-					use_this = (NULL == strstr (cvstring, valstring));
-					break;
-				case GNM_STYLE_COND_BEGINS_WITH_STR :
-					use_this = g_str_has_prefix (cvstring, valstring);
-					break;
-				case GNM_STYLE_COND_NOT_BEGINS_WITH_STR :
-					use_this = !g_str_has_prefix (cvstring, valstring);
-					break;
-				case GNM_STYLE_COND_ENDS_WITH_STR :
-					use_this = g_str_has_suffix (cvstring, valstring);
-					break;
-				case GNM_STYLE_COND_NOT_ENDS_WITH_STR :
-					use_this = !g_str_has_suffix (cvstring, valstring);
-					break;
-				}
-			}
-			value_release (val);
-		}
+		gboolean use_this = gnm_style_cond_eval (cond, cv, ep);
 
 		if (use_this) {
 			if (debug_style_conds ())
@@ -613,5 +914,6 @@ gnm_style_conditions_eval (GnmStyleConditions const *sc, GnmEvalPos const *ep)
 
 	if (debug_style_conds ())
 		g_printerr ("  No matching clauses\n");
+
 	return -1;
 }
