@@ -42,5 +42,4 @@ my $xls_filter = "$PERL -p -e '\$_ = \"\" if m{<meta:user-defined meta:name=.mso
 &test_roundtrip ($file,
 		 'format' => 'Gnumeric_Excel:xlsx',
 		 'ext' => "xlsx",
-		 'resize' => '1048576x16384',
-		 'ignore_failure' => 1);
+		 'resize' => '1048576x16384');
